(******************************************************************************)
(*                                                                            *)
(*   Folder Tools                                                             *)
(*                                                                            *)
(*   (c) 2007 Dipl.-Ing. (FH) Rainer Reusch                                   *)
(*                                                                            *)
(******************************************************************************)

unit FolderTools;

interface

uses
  Windows, Messages, SysUtils, Classes, FileCtrl, ShlObj;

type
  TFolderTools = class(TComponent)
  private
    { Private-Deklarationen }
    IsNT : boolean;
    procedure CheckPlatform;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    function GetSpecialFolder(Folder : integer) : string;
    function GetAppDir : string;
    function CreateAppDir(SubDirName : string): boolean;
    function GetCommonAppDir : string;
    function CreateCommonAppDir(SubDirName : string): boolean;
    function GetPersonalDir : string;
    function CreatePersonalDir(SubDirName : string): boolean;
  end;

const
  fldDESKTOP                 = $00; // CSIDL_DESKTOP                  Verzeichnis des Desktop des Benutzers (oberste hierarchische Ebene)
  fldINTERNET                = $01; // CSIDL_INTERNET ----
  fldPROGRAMS                = $02; // CSIDL_PROGRAMS                 Programmverzeichnis (hufig c:\Programme oder c:\Programs)
  fldCONTROLS                = $03; // CSIDL_CONTROLS                 Systemsteuerung  (Inhalt nicht darstellbar, Ergebnis: "Systemsteuerung")
  fldPRINTERS                = $04; // CSIDL_PRINTERS                 Drucker (Inhalt nicht darstellbar, Ergebnis: "Drucker")
  fldPERSONAL                = $05; // CSIDL_PERSONAL                 Verzeichnis der eigenen Dateien
  fldFAVORITES               = $06; // CSIDL_FAVORITES                Verzeichnis der Favoriten des Internet-Browsers
  fldSTARTUP                 = $07; // CSIDL_STARTUP                  Verzeichnis des Autostart-Ordners des Benutzers (Inhalt nicht darstellbar)
  fldRECENT                  = $08; // CSIDL_RECENT                   Verzeichnis "Recent", zuletzt benutztes (Inhalt nicht darstellbar)
  fldSENDTO                  = $09; // CSIDL_SENDTO                   Verzeichnis "SendTo" (kein Inhalt)
  fldBITBUCKET               = $0A; // CSIDL_BITBUCKET                Papierkorb (kein Inhalt, Ergebnis: "Papierkorb")
  fldSTARTMENU               = $0B; // CSIDL_STARTMENU                Verzeichnis des Startmens des Benutzers
  fldMY_DOCUMENTS            = $0C; // CSIDL_MYDOCUMENTS
  fldMYMUSIC                 = $0D; // CSIDL_MYMUSIC
  fldMYVIDEO                 = $0E; // CSIDL_MYVIDEO
  fldDESKTOPDIRECTORY        = $10; // CSIDL_DESKTOPDIRECTORY         Verzeichnis des Desktop, nur mit Aktenkoffer
  fldDRIVES                  = $11; // CSIDL_DRIVES                   Arbeitsplatz (alle Laufwerke, gemappte Netzlaufwerke, Systemsteuerung, Drucker; Ergebnis: Pfad oder "Arbeitsplatz" etc.)
  fldNETWORK                 = $12; // CSIDL_NETWORK                  Netzwerkumgebung (Ergebnis: UNC-Pfad oder "Netzwerkumgebung")
  fldNETHOOD                 = $13; // CSIDL_NETHOOD                  Verzeichnis der Netzwerkumgebung des Benutzers (Inhalt nicht darstellbar)
  fldFONTS                   = $14; // CSIDL_FONTS                    Verzeichnis der Schriftarten (keine Unterverzeichnisse)
  fldTEMPLATES               = $15; // CSIDL_TEMPLATES                Verzeichnis ShellNew
  fldCOMMON_STARTMENU        = $16; // CSIDL_COMMON_STARTMENU         Verzeichnis des Startmens fr alle Benutzer
  fldCOMMON_PROGRAMS         = $17; // CSIDL_COMMON_PROGRAMS          Verzeichnis von Startmen/Programme fr alle Benutzer
  fldCOMMON_STARTUP          = $18; // CSIDL_COMMON_STARTUP           Autostart-Verzeichnis fr alle Benutzer (Inhalt nicht darstellbar)
  fldCOMMON_DESKTOPDIRECTORY = $19; // CSIDL_COMMON_DESKTOPDIRECTORY  Verzeichnis des Desktop fr alle Benutzer
  fldAPPDATA                 = $1A; // CSIDL_APPDATA                  Verzeichnis der Anwendungsdaten des Benutzers
  fldPRINTHOOD               = $1B; // CSIDL_PRINTHOOD                Verzeichnis der Druckumgebung des Benutzers
  fldLOCAL_APPDATA           = $1C; // CSIDL_LOCAL_APPDATA
  fldALTSTARTUP              = $1D; // CSIDL_ALTSTARTUP  ----
  fldCOMMON_ALTSTARTUP       = $1E; // CSIDL_COMMON_ALTSTARTUP  ----        // DBCS
  fldCOMMON_FAVORITES        = $1F; // CSIDL_COMMON_FAVORITES  ----
  fldINTERNET_CACHE          = $20; // CSIDL_INTERNET_CACHE  ----
  fldCOOKIES                 = $21; // CSIDL_COOKIES  ----
  fldHISTORY                 = $22; // CSIDL_HISTORY  ----
  fldCOMMON_APPDATA          = $23; // CSIDL_COMMON_APPDATA
  fldWINDOWS                 = $24; // CSIDL_WINDOWS
  fldSYSTEM                  = $25; // CSIDL_SYSTEM
  fldPROGRAM_FILES           = $26; // CSIDL_PROGRAM_FILES
  fldMYPICTURES              = $27; // CSIDL_MYPICTURES
  fldPROFILE                 = $28; // CSIDL_PROFILE
  fldPROGRAM_FILES_COMMON    = $2B; // CSIDL_PROGRAM_FILES_COMMON
  fldCOMMON_TEMPLATES        = $2D; // CSIDL_COMMON_TEMPLATES
  fldCOMMON_DOCUMENTS        = $2E; // CSIDL_COMMON_DOCUMENTS
  fldCOMMON_ADMINTOOLS       = $2F; // CSIDL_COMMON_ADMINTOOLS
  fldADMINTOOLS              = $30; // CSIDL_ADMINTOOLS
  fldCOMMON_MUSIC            = $35; // CSIDL_COMMON_MUSIC
  fldCOMMON_PICTURES         = $36; // CSIDL_COMMON_PICTURES
  fldCOMMON_VIDEO            = $37; // CSIDL_COMMON_VIDEO
  fldCDBURN_AREA             = $3B; // CSIDL_CDBURN_AREA
  fldPROFILES                = $3E; // CSIDL_PROFILES

procedure Register;

implementation

constructor TFolderTools.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CheckPlatform;
end;

destructor TFolderTools.Destroy;
begin
  inherited Destroy;
end;

procedure TFolderTools.CheckPlatform;
// Windows-Plattform ermitteln
// Ergebnis in IsNT
var
  vi : TOSVersionInfo;
begin
  // Windows-Version
  vi.dwOSVersionInfoSize:=SizeOf(vi);
  GetVersionEx(vi);
  IsNT:=vi.dwPlatformId=VER_PLATFORM_WIN32_NT;
end;

function TFolderTools.GetSpecialFolder(Folder : integer) : string;
var
  ppidl : PITEMIDLIST;
begin
  Result:=EmptyStr;
  if SUCCEEDED(SHGetSpecialFolderLocation(0,Folder,ppidl)) then
  begin
    SetLength(Result,MAX_PATH);
    SHGetPathFromIDList(ppidl,PChar(Result));
    SetLength(Result,StrLen(PChar(Result)));
  end;
end;

function TFolderTools.GetAppDir : string;
begin
  Result:=GetSpecialFolder(fldAPPDATA)+'\';
end;

function TFolderTools.CreateAppDir(SubDirName : string): boolean;
var
  s : string;
begin
  s:=GetAppDir+SubDirName;
  if Length(s)>1 then Result:=SysUtils.ForceDirectories(s)
                 else Result:=false;
end;

function TFolderTools.GetCommonAppDir : string;
begin
  Result:=GetSpecialFolder(fldCOMMON_APPDATA)+'\';
  if Length(Result)<2 then
  begin  // kein Verzeichnis definiert
    if IsNT then
    begin  // NT 4.0 liefert hier kein Verzeichnis
      Result:=ExtractFilePath(GetSpecialFolder(fldCOMMON_DESKTOPDIRECTORY));
    end
    else Result:=GetAppDir;  // Windows 9.x kennt keine Beutzer
  end;
end;

function TFolderTools.CreateCommonAppDir(SubDirName : string): boolean;
var
  s : string;
begin
  s:=GetCommonAppDir+SubDirName;
  if Length(s)>1 then Result:=SysUtils.ForceDirectories(s)
                 else Result:=false;
end;

function TFolderTools.GetPersonalDir : string;
begin
  Result:=GetSpecialFolder(fldPERSONAL)+'\';
end;

function TFolderTools.CreatePersonalDir(SubDirName : string): boolean;
var
  s : string;
begin
  s:=GetPersonalDir+SubDirName;
  if Length(s)>1 then Result:=SysUtils.ForceDirectories(s)
                 else Result:=false;
end;

procedure Register;
begin
  RegisterComponents('Toolbox', [TFolderTools]);
end;

end.
