/****************************************************************************

USB-CAN-Interface Communication Center
Kommunikation mit dem Gert

(c) 2020 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik

RADStudio XE7

Erstellt: 2020-03-11
Letzte nderung: 2020-03-14

*****************************************************************************/

//---------------------------------------------------------------------------

#ifndef UnitCommH
#define UnitCommH

#include <System.Classes.hpp>
#include "uci_common.h"
#include "uci_if.h"

//---------------------------------------------------------------------------

// Klasse zur Kommunikation mit dem Gert (ber einen Thread)
class TDevComm : public TThread
{
private:
	TUCI *uci;
	AnsiString SerialNo;
	bool Finished;
	TDeviceInfo FDeviceInfo;  // Gerteinformationen (cmd=USB_INFO, wenn Informationen verfgbar sind)
	TDeviceConfig FDeviceConfig;	// neue zu setzende Gertekonfiguration
	TCANFilter FCANFilter;				// neu zu setzender CAN-Filter
	TList *TxQueue;  					// Sendewarteschlange
	TList *RxQueue;  					// Empfangswarteschlange
	TCANData *rd;    					// im Heap vorgehaltener Puffer fr den Empfang eines Datensatzes
	TCANData *td;							// neue CAN-Botschaft, die in Sendewarteschlange gesetzt werden soll
	TCANData *FReceived;  		// bereitgestellter empfangener Datensatz (wenn RxFlag==2)
	bool TxFlag;							// eine neue CAN-Botschaft soll zur Sendewarteschlange hinzugefgt werden
	int RxFlag;								// FReceived: 0=keine Daten vorhanden, 1=vorhandene Daten wurden ausgelesen, 2=Daten vorhanden (und noch nicht ausgelesen)
	int CmdFlag;							// auszufhrendes Kommando (USB_-Konstante), USB_NONE bei keinem Kommando
protected:
	virtual void __fastcall Execute(void);
public:
	__property TDeviceInfo DeviceInfo = { read=FDeviceInfo };
	__fastcall TDevComm(TUCI *AnUCI, AnsiString ASerialNo);
	__fastcall ~TDevComm(void);
	void __fastcall Transmit(TCANData *ACanData);
	bool __fastcall Receive(TCANData *ACanData);
	void __fastcall SetDeviceConfig(TDeviceConfig *ADeviceConfig);
	void __fastcall SetFilter(TCANFilter *ACANFilter);
	void __fastcall ClearStatus(void);
};

#endif
