/****************************************************************************

USB-CAN-Interface Communication Center
Dialog: Filtereinstellungen

(c) 2019-2020 Reusch Elektronik, Dipl.-Ing. (FH) Rainer Reusch

Created: 2020-03-14
Changed: 2020-03-14

*****************************************************************************/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitFilter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TFormFilter *FormFilter;

//---------------------------------------------------------------------------
__fastcall TFormFilter::TFormFilter(TComponent* Owner)
	: TForm(Owner)
//---------------------------------------------------------------------------
{
	//
}

//---------------------------------------------------------------------------
void __fastcall TFormFilter::FormCreate(TObject *Sender)
//---------------------------------------------------------------------------
{
	ClientWidth = BitBtnCancel->Left+BitBtnCancel->Width+BitBtnDefault->Left;
	ClientHeight = BitBtnCancel->Top+BitBtnCancel->Height+BitBtnDefault->Left;
}

//---------------------------------------------------------------------------
bool __fastcall TFormFilter::Execute(TPoint WinPos, TCANFilters *ACANFilters)
//---------------------------------------------------------------------------
{
	bool r = false;
	memcpy_s(&CANFilters,sizeof(TCANFilters),ACANFilters,sizeof(TCANFilters));
	Left = WinPos.x;
	Top = WinPos.y;
	LastIndex = TabControl1->TabIndex;
	FilterToDlg();
	if (ShowModal()==mrOk) {
		DlgToFilter();
		memcpy_s(ACANFilters,sizeof(TCANFilters),&CANFilters,sizeof(TCANFilters));
		r = true;
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TFormFilter::FilterToDlg(void)
//---------------------------------------------------------------------------
// Filtereinstellungen in Dialog bertragen
{
	int i = TabControl1->TabIndex;
	CheckBoxActive->Checked = CANFilters[i].Active;
	ComboBoxIDType->ItemIndex = CANFilters[i].IdType;
	ComboBoxMode->ItemIndex = CANFilters[i].Mode;
	LabeledEdit1->Text = IntToHex((int)CANFilters[i].ID,1);
	LabeledEdit2->Text = IntToHex((int)CANFilters[i].Mask,1);
	switch (ComboBoxMode->ItemIndex) {
	case 0:
		LabeledEdit1->EditLabel->Caption = "Identifier (hex)";
		LabeledEdit2->EditLabel->Caption = "Mask (hex)";
		break;
	case 1:
		LabeledEdit1->EditLabel->Caption = "1. Identifier (hex)";
		LabeledEdit2->EditLabel->Caption = "2. Identifier (hex)";
		break;
	default:
		LabeledEdit1->EditLabel->Caption = "(hex)";
		LabeledEdit2->EditLabel->Caption = "(hex)";
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormFilter::DlgToFilter(void)
//---------------------------------------------------------------------------
// Dialogeingaben bernehmen
{
	const int Modes[2] = {CAN_FILTERMODE_IDMASK,CAN_FILTERMODE_IDLIST};
	const int Types[2] = {CAN_ID_STD,CAN_ID_EXT};
	CANFilters[LastIndex].Active = CheckBoxActive->Checked;
	CANFilters[LastIndex].Mode = Modes[ComboBoxMode->ItemIndex];
	CANFilters[LastIndex].IdType = Types[ComboBoxIDType->ItemIndex];
	CANFilters[LastIndex].ID = FormEditMsg->StrToHex(LabeledEdit1->Text);
	CANFilters[LastIndex].Mask = FormEditMsg->StrToHex(LabeledEdit2->Text);
	LastIndex = TabControl1->TabIndex;
}

//---------------------------------------------------------------------------
void __fastcall TFormFilter::BitBtnDefaultClick(TObject *Sender)
//---------------------------------------------------------------------------
// Grundeinstellung setzen
{
	for (int i=0; i<14; i++) {
		CANFilters[i].Bank = i;
		if (i) CANFilters[i].Active = 0;
		else CANFilters[i].Active = 1;
		CANFilters[i].Mode = CAN_FILTERMODE_IDMASK;
		CANFilters[i].IdType = CAN_ID_STD;
		CANFilters[i].ID = 0;
		CANFilters[i].Mask = 0;
	}
	TabControl1->TabIndex = 0;
	LastIndex = 0;
	FilterToDlg();  // Dialog aktualisieren
}

//---------------------------------------------------------------------------
void __fastcall TFormFilter::TabControl1Change(TObject *Sender)
//---------------------------------------------------------------------------
// andere Filterbank wurde ausgewhlt
{
	DlgToFilter();  // Eingaben bernehmen
	FilterToDlg();  // Dialog aktualisieren
}

//---------------------------------------------------------------------------
