/****************************************************************************

USB-CAN-Interface Communication Center
Hauptfenster

(c) 2020 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik

RADStudio XE7

Erstellt: 2020-03-09
Letzte nderung: 2020-05-02

*****************************************************************************/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

#define MIN_PANEL_RX_HEIGHT PanelFunction->Height+Splitter1->MinSize+Splitter1->Height+PanelTx->Height+StatusBar1->Height+80;
#define STD "Std"
#define EXT "Ext"

const UnicodeString RequestStr = "remote request (DLC=";

TFormMain *FormMain;

//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
	: TForm(Owner)
//---------------------------------------------------------------------------
{
	cfg_Init((TForm *)&FormMain);
	uci = new TUCI();
	DevComm = NULL;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
//---------------------------------------------------------------------------
// Initialization
{
	Initialized = false;
	ListTx = new TList();

	// Fenster
	Left   = cfg->ReadInteger(SEC_MAIN,PAR_X,10);
	Top    = cfg->ReadInteger(SEC_MAIN,PAR_Y,10);
	Width  = cfg->ReadInteger(SEC_MAIN,PAR_W,800);
	Height = cfg->ReadInteger(SEC_MAIN,PAR_H,600);
	WindowState = (TWindowState)cfg->ReadInteger(SEC_MAIN,PAR_S,(int)wsNormal);

	PanelTx->Top = cfg->ReadInteger(SEC_MAIN,PAR_V,(ClientHeight>>1)-PanelFunction->Height-StatusBar1->Height);
	PanelTx->Height = ClientHeight-PanelTx->Top-StatusBar1->Height;
	Application->ProcessMessages();
	PanelRx->Height = 1;
	PanelTx->Align = alBottom;
	Splitter1->Align = alBottom;
	PanelRx->Align = alClient;
	Constraints->MinWidth = SpeedButtonExit->Left+SpeedButtonExit->Width+20;
	Constraints->MinHeight = MIN_PANEL_RX_HEIGHT;
	ComboBoxView->ItemIndex = cfg->ReadInteger(SEC_MAIN,PAR_N,0);

	LoadFromFile(cfg);

	RxCount = 0;
	TxCount = 0;
	Initialized = true;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCloseQuery(TObject *Sender, bool &CanClose)
//---------------------------------------------------------------------------
{
	//
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormClose(TObject *Sender, TCloseAction &Action)
//---------------------------------------------------------------------------
{
	// Fenster
	cfg->WriteInteger(SEC_MAIN,PAR_S,(int)WindowState);
	if (WindowState==wsNormal) {
		cfg->WriteInteger(SEC_MAIN,PAR_X,Left);
		cfg->WriteInteger(SEC_MAIN,PAR_Y,Top);
		cfg->WriteInteger(SEC_MAIN,PAR_W,Width);
		cfg->WriteInteger(SEC_MAIN,PAR_H,Height);
	}
	cfg->WriteInteger(SEC_MAIN,PAR_V,PanelTx->Top);
	cfg->WriteInteger(SEC_MAIN,PAR_N,ComboBoxView->ItemIndex);

	SaveToFile(cfg);
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
//---------------------------------------------------------------------------
{
	TCANData *cd;
	while (ListTx->Count) {
		cd = (TCANData *)ListTx->Items[0];
		delete cd;
		ListTx->Items[0] = NULL;
		ListTx->Delete(0);
	}
	delete ListTx;
	if (DevComm) delete DevComm;
	delete uci;
	cfg_DeInit();
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
//---------------------------------------------------------------------------
{
	//
}

//---------------------------------------------------------------------------
MESSAGE void __fastcall TFormMain::WMDeviceChange(TMessage &Message)
//---------------------------------------------------------------------------
// Ein Gert wurde angeschlossen oder entfernt
// Prfen, ob die verwendete Checkbox noch angeschlossen ist
{
	if (uci) {
		if ((uci->Opened) && (uci->DeviceRemoved())) {  // das Gert war geffnet und wurde entfernt
			Disconnect();
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::LoadFromFile(TIniFile *f)
//---------------------------------------------------------------------------
// Konfigurationsdaten aus einer Datei laden
{
	// Gertekonfiguration
	DeviceConfig.Speed = f->ReadInteger(SEC_DEVCFG,PAR_SPEED,SPEED_10K);
	DeviceConfig.Termination = f->ReadInteger(SEC_DEVCFG,PAR_TERM,1);
	DeviceConfig.Retransmit = f->ReadInteger(SEC_DEVCFG,PAR_RETRM,1);
	DeviceConfig.LoopBackMode = f->ReadInteger(SEC_DEVCFG,PAR_LOOPBK,0);

	// Filter
	for (int i=0; i<14; i++) {
		CANFilters[i].Bank = i;
		UnicodeString s = SEC_FILTER+IntToStr(i);
		if (i) CANFilters[i].Active = f->ReadInteger(s,PAR_ACTIVE,0);
		else CANFilters[i].Active = f->ReadInteger(s,PAR_ACTIVE,1);
		CANFilters[i].Mode = f->ReadInteger(s,PAR_MODE,CAN_FILTERMODE_IDMASK);
		CANFilters[i].IdType = f->ReadInteger(s,PAR_IDTYPE,CAN_ID_STD);
		CANFilters[i].ID = f->ReadInteger(s,PAR_ID,0);
		CANFilters[i].Mask = f->ReadInteger(s,PAR_MASK,0);
	}

	// erstellte CAN-Botschaften
	int n = f->ReadInteger(SEC_MSG,PAR_N,0);
	UnicodeString s;
	if (n) {
		ListTx->Clear();
		ListBoxTx->Clear();
		for (int i=0; i<n; i++) {
			s = f->ReadString(SEC_MSG,IntToStr(i),"");
			ListBoxTx->Items->Add(s);
			TCANData *cd = new TCANData;
			StrToData(s,cd);
			ListTx->Add(cd);
		}
		ListBoxTx->ItemIndex = ListBoxTx->Items->Count-1;
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::SaveToFile(TIniFile *f)
//---------------------------------------------------------------------------
// Konfigurationsdaten in eine Datei speichern
{
	// Gertekonfiguration
	f->WriteInteger(SEC_DEVCFG,PAR_SPEED,DeviceConfig.Speed);
	f->WriteInteger(SEC_DEVCFG,PAR_TERM,DeviceConfig.Termination);
	f->WriteInteger(SEC_DEVCFG,PAR_RETRM,DeviceConfig.Retransmit);
	f->WriteInteger(SEC_DEVCFG,PAR_LOOPBK,DeviceConfig.LoopBackMode);

	// Filter
	for (int i=0; i<14; i++) {
		UnicodeString s = SEC_FILTER+IntToStr(i);
		f->WriteInteger(s,PAR_ACTIVE,CANFilters[i].Active);
		f->WriteInteger(s,PAR_MODE,CANFilters[i].Mode);
		f->WriteInteger(s,PAR_IDTYPE,CANFilters[i].IdType);
		f->WriteInteger(s,PAR_ID,CANFilters[i].ID);
		f->WriteInteger(s,PAR_MASK,CANFilters[i].Mask);
	}

	// Erstellte Botschaften
	f->WriteInteger(SEC_MSG,PAR_N,ListBoxTx->Items->Count);
	if (ListBoxTx->Items->Count) {
		for (int i=0; i<ListBoxTx->Items->Count; i++) {
			f->WriteString(SEC_MSG,IntToStr(i),ListBoxTx->Items->Strings[i]);
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::MsgCount(void)
//---------------------------------------------------------------------------
// Anzeige der Anzahl empfangener und gesendeter Botschaften in der Statuszeile
{
	if (DevComm) {	// Verbindung zum Gert hergestellt
		StatusBar1->Panels->Items[1]->Text = "Rx: "+IntToStr(RxCount);
		StatusBar1->Panels->Items[2]->Text = "Tx: "+IntToStr(TxCount);
	} else {	// Verbindung zum Gert getrennt
		StatusBar1->Panels->Items[1]->Text = "";
		StatusBar1->Panels->Items[2]->Text = "";
	}
}

//---------------------------------------------------------------------------
UnicodeString __fastcall TFormMain::DataToStr(UnicodeString Prolog, TCANData *ACANData)
//---------------------------------------------------------------------------
// CAN-Datensatz in String umwandeln
{
	UnicodeString r;
	if (Prolog.Length()) r = Prolog+"\t";
	if (ACANData->IdType==CAN_ID_STD) {
		r = r+STD+"\t";
		r = r+"  "+IntToHex((int)ACANData->Identifier,3);
	} else {
		r = r+EXT+"\t";
		r = r+IntToHex((int)ACANData->Identifier,5);
	}
	if (ACANData->RemoteRequest) {
		r = r+"\t"+RequestStr+IntToStr(ACANData->DataLength)+")";
	} else {
		int i = 0;
		while (i<ACANData->DataLength) {
			r = r+"\t"+IntToHex((int)(ACANData->Data[i]),2);
			i++;
		}
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::StrToData(UnicodeString AStr, TCANData *ACANData)
//---------------------------------------------------------------------------
// String (ohne Prolog) in CAN-Datensatz umwandeln
{
	int p, i = 0;
	UnicodeString s, ss;
	ACANData->cmd = USB_CAN_TX;
	ACANData->DataLength = 0;
	ACANData->Identifier = 0;
	for (p=0; p<8; p++) ACANData->Data[p] = 0;
	do {
		p = AStr.Pos("\t");
		if (p) {
			s = AStr.SubString(1,p-1);
			AStr.Delete(1,p);
		} else {
			s = AStr;
		}
		if (s.Length()) {
			switch (i) {
			case 0 :  // Standard oder Extended
				if (s==STD) ACANData->IdType = CAN_ID_STD;
				else ACANData->IdType = CAN_ID_EXT;
				break;
			case 1 :	// Identifier
				ACANData->Identifier = FormEditMsg->StrToHex(s);
				break;
			case 2 :
				ss = s.SubString(1,RequestStr.Length());
				if (ss==RequestStr) {
					ACANData->RemoteRequest = CAN_RTR_REMOTE;
					for (int j=0; j<8; j++) ACANData->Data[j] = 0;
					ss = s.SubString(RequestStr.Length()+1,1);
					try {
						ACANData->DataLength = StrToInt(ss);
					} catch (Exception &exception) {
						ACANData->DataLength = 0;
					}
					i = 10;
				} else {
					ACANData->RemoteRequest = CAN_RTR_DATA;
					try {
						ACANData->Data[ACANData->DataLength] = FormEditMsg->StrToHex(s);
						ACANData->DataLength++;
					} catch (Exception &exception) {
					}
				}
				break;
			case 3 :  // Daten
			case 4 :
			case 5 :
			case 6 :
			case 7 :
			case 8 :
			case 9 :
				ACANData->Data[ACANData->DataLength] = FormEditMsg->StrToHex(s);
				ACANData->DataLength++;
				break;
			default:
				;
			}
			i++;
		}
	} while (p);
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::Splitter1Moved(TObject *Sender)
//---------------------------------------------------------------------------
{
	Constraints->MinHeight = MIN_PANEL_RX_HEIGHT;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::Disconnect(void)
//---------------------------------------------------------------------------
// Verbindung zum Gert trennen
{
	if (DevComm) {	// Kommunikations-Instanz wurde angelegt -> Verbindung ist hergestellt
		TimerReceive->Enabled = false;
		delete DevComm;
		DevComm = NULL;
		SpeedButtonConnect->Caption = "&Connect";
		StatusBar1->Panels->Items[0]->Text = "disconnected";
  }
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonConnectClick(TObject *Sender)
//---------------------------------------------------------------------------
// Verbinden/trennen
{
	if (DevComm) {	// Kommunikations-Instanz wurde angelegt -> Verbindung ist hergestellt
		Disconnect();
	} else {	// Verbindung zu einem Gert herstellen
		TPoint wp;
		wp.x = Left+20;
		wp.y = Top+20;
		AnsiString sn = FormConnect->Execute(wp,uci);
		if (sn.Length()) {
			DevComm = new TDevComm(uci,sn);
			if (uci->Opened) {	// Verbindung zum Gert hergestellt (erfolgte in DevComm)
				SpeedButtonConnect->Caption = "Dis&connect";
				RxCount = 0;
				TxCount = 0;
				StatusBar1->Panels->Items[0]->Text = "connected";
				MsgCount();
				DevComm->SetDeviceConfig(&DeviceConfig);  // Gert konfigurieren
				for (int i=0; i<14; i++) {
					DevComm->SetFilter(&(CANFilters[i]));  // die einzelnen Filter setzen
				}
				DevComm->ClearStatus();  // Status zurck setzen
				TimerReceive->Enabled = true;
			} else {	// Herstellung der Verbindung gescheitert
				delete DevComm;
				DevComm = NULL;
				MessageBeep(16);
				MessageDlg("Establish connection to selected device failed!",mtError,TMsgDlgButtons() << mbCancel,0);
      }
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonSetupClick(TObject *Sender)
//---------------------------------------------------------------------------
// Konfigurationsdialog
{
	if (DevComm==NULL) return;
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	if (FormDeviceConfig->Execute(wp,&DeviceConfig)) {
		DevComm->SetDeviceConfig(&DeviceConfig);
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonFilterClick(TObject *Sender)
//---------------------------------------------------------------------------
// Filterdialog
{
	if (DevComm==NULL) return;
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	if (FormFilter->Execute(wp,&CANFilters)) {
		for (int i=0; i<14; i++) {
			DevComm->SetFilter(&(CANFilters[i]));  // die einzelnen Filter setzen
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonExitClick(TObject *Sender)
//---------------------------------------------------------------------------
// Programm beenden
{
	Close();
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::mi_AboutClick(TObject *Sender)
//---------------------------------------------------------------------------
// ber das Programm
{
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	FormAbout->Execute(wp);
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxViewCloseUp(TObject *Sender)
//---------------------------------------------------------------------------
// Auswahl der knftigen Anzeige von Botschaften
{
	//
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnRxClearClick(TObject *Sender)
//---------------------------------------------------------------------------
// Liste der Botschaften lschen
{
	ListBoxRx->Clear();
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnNewClick(TObject *Sender)
//---------------------------------------------------------------------------
// Neuer Sendedatensatz
{
	TCANData *cd = new TCANData;
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	if (FormEditMsg->ExecuteNew(wp,cd)) {
		ListTx->Add(cd);
		ListBoxTx->Items->Add(DataToStr("",cd));
		ListBoxTx->ItemIndex = ListBoxTx->Items->Count-1;
	} else delete cd;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnTxEditClick(TObject *Sender)
//---------------------------------------------------------------------------
// Sendedatensatz bearbeiten
{
	if (ListBoxTx->ItemIndex<0) return;
	TCANData *cd = (TCANData *)ListTx->Items[ListBoxTx->ItemIndex];
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	if (FormEditMsg->ExecuteEdit(wp,cd)) {
		ListBoxTx->Items->Strings[ListBoxTx->ItemIndex] = DataToStr("",cd);
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnDeleteClick(TObject *Sender)
//---------------------------------------------------------------------------
// Sendedatensatz lschen
{
	int i = ListBoxTx->ItemIndex;
	if (i<0) return;
	TCANData *cd = (TCANData *)ListTx->Items[i];
	delete cd;
	ListTx->Items[i] = NULL;
	ListTx->Delete(i);
	ListBoxTx->Items->Delete(i);
	ListBoxTx->ItemIndex = i-1;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnTxClick(TObject *Sender)
//---------------------------------------------------------------------------
// ausgewhlten Sendedatensatz senden
{
	if ((ListBoxTx->ItemIndex<0) || (DevComm==NULL)) return;
	TCANData *td = new TCANData;
	TCANData *cd = (TCANData *)ListTx->Items[ListBoxTx->ItemIndex];
	memcpy_s(td,sizeof(TCANData),cd,sizeof(TCANData));
	DevComm->Transmit(td);
	TxCount++;
	if ((ComboBoxView->ItemIndex==0) || (ComboBoxView->ItemIndex==2)) {
		ListBoxRx->Items->Add(DataToStr("Tx",td));
		ListBoxRx->ItemIndex = ListBoxRx->Items->Count-1;
	}
	MsgCount();
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::BitBtnTxAllClick(TObject *Sender)
//---------------------------------------------------------------------------
// alle Sendedatenstze senden
{
	if ((ListBoxTx->Count==0) || (DevComm==NULL)) return;
	for (int i=0; i<ListBoxTx->Count; i++) {
		TCANData *td = new TCANData;
		TCANData *cd = (TCANData *)ListTx->Items[i];
		memcpy_s(td,sizeof(TCANData),cd,sizeof(TCANData));
		DevComm->Transmit(td);
		TxCount++;
		if ((ComboBoxView->ItemIndex==0) || (ComboBoxView->ItemIndex==2)) {
			ListBoxRx->Items->Add(DataToStr("Tx",td));
			ListBoxRx->ItemIndex = ListBoxRx->Items->Count-1;
		}
	}
	MsgCount();
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::mf_LoadClick(TObject *Sender)
//---------------------------------------------------------------------------
// Einstellungen laden
{
	if (OpenDialog1->Execute(Handle)) {
		TIniFile *f = new TIniFile(OpenDialog1->FileName);
		LoadFromFile(f);
		delete f;
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::mf_SaveClick(TObject *Sender)
//---------------------------------------------------------------------------
// Einstellungen speichern
{
	if (OpenDialog1->FileName.Length()) {
		TIniFile *f = new TIniFile(OpenDialog1->FileName);
		SaveToFile(f);
		delete f;
	} else mf_SaveAsClick(Sender);
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::mf_SaveAsClick(TObject *Sender)
//---------------------------------------------------------------------------
// Einstellungen speichern unter
{
	if (SaveDialog1->Execute(Handle)) {
		bool DoIt = true;
		if (FileExists(SaveDialog1->FileName,false)) {
			MessageBeep(0);
			DoIt = MessageDlg("The file\n"+SaveDialog1->FileName+"\nexists! Overwrite?",mtConfirmation,TMsgDlgButtons() << mbYes << mbNo,0)==mrYes;
		}
		if (DoIt) {
			TIniFile *f = new TIniFile(SaveDialog1->FileName);
			SaveToFile(f);
			delete f;
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::mi_DeviceInfoClick(TObject *Sender)
//---------------------------------------------------------------------------
// Dialog Gerteinformationen
{
	TPoint wp;
	wp.x = Left+20;
	wp.y = Top+20;
	FormDeviceInfo->Execute(wp,DevComm);
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::TimerReceiveTimer(TObject *Sender)
//---------------------------------------------------------------------------
{
	TCANData cd;
	if (DevComm==NULL) return;
	if (DevComm->Receive(&cd)) {  // es liegen gltige Daten vor
		RxCount++;
		if (ComboBoxView->ItemIndex<2) {
			ListBoxRx->Items->Add(DataToStr("Rx",&cd));
			ListBoxRx->ItemIndex = ListBoxRx->Items->Count-1;
		}
		MsgCount();
	}
}

//---------------------------------------------------------------------------

