/****************************************************************************

USB-CAN-Interface Communication Center
Hauptfenster

(c) 2020 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik

RADStudio XE7

Erstellt: 2020-03-09
Letzte nderung: 2020-03-09

*****************************************************************************/

//---------------------------------------------------------------------------

#ifndef UnitMainH
#define UnitMainH
//---------------------------------------------------------------------------
#include <System.Classes.hpp>
#include <Vcl.Controls.hpp>
#include <Vcl.StdCtrls.hpp>
#include <Vcl.Forms.hpp>
#include <Vcl.Buttons.hpp>
#include <Vcl.ComCtrls.hpp>
#include <Vcl.ExtCtrls.hpp>
#include <Vcl.Menus.hpp>
#include "UnitConfig.h"
#include "UnitConnect.h"
#include "uci_common.h"
#include "uci_if.h"
#include "UnitEditMsg.h"
#include "UnitComm.h"
#include "UnitDeviceInformation.h"
#include "UnitDeviceConfig.h"
#include "UnitFilter.h"
#include "UnitAbout.h"
#include <Vcl.Dialogs.hpp>
//---------------------------------------------------------------------------

class TFormMain : public TForm
{
__published:	// Von der IDE verwaltete Komponenten
	TStatusBar *StatusBar1;
	TMainMenu *MainMenu1;
	TPanel *PanelFunction;
	TSpeedButton *SpeedButtonConnect;
	TSpeedButton *SpeedButtonSetup;
	TSpeedButton *SpeedButtonFilter;
	TSpeedButton *SpeedButtonExit;
	TMenuItem *mm_File;
	TMenuItem *mm_Info;
	TMenuItem *N1;
	TMenuItem *mf_Exit;
	TMenuItem *mi_About;
	TPanel *PanelRx;
	TSplitter *Splitter1;
	TPanel *PanelTx;
	TPanel *PanelRxTop;
	TPanel *PanelRxTopRight;
	TComboBox *ComboBoxView;
	TBitBtn *BitBtnRxClear;
	TPanel *PanelTxTop;
	TPanel *PanelTxTopRight;
	TListBox *ListBoxRx;
	TPanel *PanelRxFunction;
	TPanel *PanelTxFunctionLeft;
	TPanel *PanelTxFunctionRight;
	TBitBtn *BitBtnNew;
	TBitBtn *BitBtnTxEdit;
	TBitBtn *BitBtnDelete;
	TBitBtn *BitBtnTx;
	TBitBtn *BitBtnTxAll;
	TListBox *ListBoxTx;
	TMenuItem *mi_DeviceInfo;
	TTimer *TimerReceive;
	TMenuItem *mf_Load;
	TMenuItem *mf_Save;
	TMenuItem *mf_SaveAs;
	TOpenDialog *OpenDialog1;
	TSaveDialog *SaveDialog1;
	void __fastcall FormCreate(TObject *Sender);
	void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
	void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
	void __fastcall FormDestroy(TObject *Sender);
	void __fastcall FormResize(TObject *Sender);
	void __fastcall SpeedButtonConnectClick(TObject *Sender);
	void __fastcall SpeedButtonSetupClick(TObject *Sender);
	void __fastcall SpeedButtonFilterClick(TObject *Sender);
	void __fastcall SpeedButtonExitClick(TObject *Sender);
	void __fastcall mi_AboutClick(TObject *Sender);
	void __fastcall Splitter1Moved(TObject *Sender);
	void __fastcall ComboBoxViewCloseUp(TObject *Sender);
	void __fastcall BitBtnRxClearClick(TObject *Sender);
	void __fastcall BitBtnNewClick(TObject *Sender);
	void __fastcall BitBtnTxEditClick(TObject *Sender);
	void __fastcall BitBtnDeleteClick(TObject *Sender);
	void __fastcall BitBtnTxClick(TObject *Sender);
	void __fastcall BitBtnTxAllClick(TObject *Sender);
	void __fastcall mi_DeviceInfoClick(TObject *Sender);
	void __fastcall TimerReceiveTimer(TObject *Sender);
	void __fastcall mf_LoadClick(TObject *Sender);
	void __fastcall mf_SaveClick(TObject *Sender);
	void __fastcall mf_SaveAsClick(TObject *Sender);
private:	// Benutzer-Deklarationen
	bool Initialized;
	TUCI *uci;  // Gerte-Instanz des ausgewhlten USB-CAN-Interfaces
	TDevComm *DevComm;  // Instanz fr die Kommunikation mit dem Gert
	TDeviceConfig DeviceConfig;	// aktuelle Gertekonfiguration
	TCANFilters CANFilters;	// die einzelnen CAN-Filter
	TList *ListTx;  // Liste der vorgehaltenen Botschaften zum senden (TCANData)
	int RxCount, TxCount;	// Anzahl empfangener und gesendeter Botschaften
	void __fastcall MsgCount(void);
	void __fastcall LoadFromFile(TIniFile *f);
	void __fastcall SaveToFile(TIniFile *f);
	UnicodeString __fastcall DataToStr(UnicodeString Prolog, TCANData *ACANData);
	void __fastcall StrToData(UnicodeString AStr, TCANData *ACANData);
	void __fastcall Disconnect(void);
	MESSAGE void __fastcall WMDeviceChange(TMessage &Message);
protected:
	BEGIN_MESSAGE_MAP
		MESSAGE_HANDLER(WM_DEVICECHANGE, TMessage, WMDeviceChange);
	END_MESSAGE_MAP(TForm)
public:		// Benutzer-Deklarationen
	__fastcall TFormMain(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern PACKAGE TFormMain *FormMain;
//---------------------------------------------------------------------------
#endif
